#!/usr/local/bin/perl
#
# detect_and_draw.cgi
#

use strict;
use warnings;
# $PERL5LIB
use lib qw(
  /home/hoge/local/lib/perl5/site_perl/5.8.8/mach
  /home/hoge/local/lib/perl5/site_perl/5.8.8
);
use CGI ':cgi';
use CGI::Carp 'fatalsToBrowser';
use Imager;
use Imager::AnimeFace qw/detect_animeface/;
use LWP::UserAgent;
use HTTP::Cache::Transparent;
use Cache::File;
use constant  FONT_FILE => './font.ttf';# font file
use constant  ERROR_FILE => './error.png';# error image

HTTP::Cache::Transparent::init( {
    BasePath => './tmp',
    MaxAge    => 8*24,
    NoUpdate  => 3600 * 2
});

my $query = CGI->new();
my $image_url = $query->param('image_url');
my $site_url = $query->param('site_url');
my $step = $query->param('step') || 4.0;
my $scale_factor = $query->param('scale_factor') || 1.095;
my $window = $query->param('window') || 42.592;

unless ($image_url) {
    error_output();
    exit(0);
} elsif ($image_url =~ /$ENV{REQUEST_URI}/) {
    error_output();
    exit(0);
} else {
    my $im = detect_and_draw(
	$step, $scale_factor, $window,
	$image_url, $site_url
    );
    if ($im) {
	print CGI::header(-type => 'image/png');
	binmode(STDOUT);
	$im->write(fd => fileno(STDOUT), type=>'png');
    } else {
	error_output();
	exit(0);
    }
}


sub error_output
{
    my $im = Imager->new();
    $im->read(file => ERROR_FILE);
    print header(-type => 'image/png');
    binmode(STDOUT);
    $im->write(fd => fileno(STDOUT), type=>'png');
}

sub detect_and_draw
{
    my ($step, $scale_factor, $window, $image_url, $site_url) = @_;
    my $ua = LWP::UserAgent->new();
    my $req = HTTP::Request->new(GET => $image_url);
    my $im = Imager->new();
    my $results;
    my $res;

    if ($site_url) {
	$req->referer($site_url);
    }
    $ua->max_size(2 * 1024 * 1024);
    $res = $ua->request($req);

    if (!$res->is_success) {
	# error
	return undef;
    }
    if (!$im->read(data => $res->content)) {
	return undef;
    }
    $results = detect_animeface(
	$im,
	step => $step, scale_factor => $scale_factor, window => $window
    );
    draw($im, $results);
    
    return $im;
}

sub minv
{
    my($a, $b) = @_;
    return $a < $b ? $a:$b;
}

sub draw
{
    my ($im, $results) = @_;
    my $black = Imager::Color->new(r => 0, g => 0, b => 0);
    my $font = Imager::Font->new(file => FONT_FILE);
    
    foreach my $face (@{$results}) {
	my $score_grad = ($face->{likelihood}-0.5) / 0.5;
	my $score_color = Imager::Color->new(
	    r => 50  * $score_grad,
	    g => 255 * $score_grad,
	    b => 200 * $score_grad
	    );
	my $scale = $face->{face}->{width}/32.0;

	$im->string(
            text => sprintf("%.4f", $face->{likelihood}),
            color => $black,
	    size=> 14,
	    font=>$font,
	    aa => 1,
	    x => $face->{face}->{x}+1,
	    y => ($face->{face}->{y}));
	$im->string(
            text => sprintf("%.4f", $face->{likelihood}),
            color => $score_color,
	    size=> 14,
	    font=>$font,
	    aa => 1,
	    x => $face->{face}->{x},
	    y => ($face->{face}->{y}-1));
	$im->box(
	    xmin => $face->{face}->{x} + 1,
	    ymin => $face->{face}->{y} + 1,
	    xmax => $face->{face}->{x} + $face->{face}->{width} + 1,
	    ymax => $face->{face}->{y} + $face->{face}->{height} + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{face}->{x},
	    ymin => $face->{face}->{y},
	    xmax => $face->{face}->{x} + $face->{face}->{width},
	    ymax => $face->{face}->{y} + $face->{face}->{height},
	    color => $score_color
	);
	# eyes
	$im->box(
	    xmin => $face->{eyes}->{left}->{x} + 1,
	    ymin => $face->{eyes}->{left}->{y} + 1,
	    xmax => $face->{eyes}->{left}->{x} + $face->{eyes}->{left}->{width} + 1,
	    ymax => $face->{eyes}->{left}->{y} + $face->{eyes}->{left}->{height} + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{eyes}->{left}->{x},
	    ymin => $face->{eyes}->{left}->{y},
	    xmax => $face->{eyes}->{left}->{x} + $face->{eyes}->{left}->{width},
	    ymax => $face->{eyes}->{left}->{y} + $face->{eyes}->{left}->{height},
	    color => $score_color
	);
	$im->box(
	    xmin => $face->{eyes}->{right}->{x} + 1,
	    ymin => $face->{eyes}->{right}->{y} + 1,
	    xmax => $face->{eyes}->{right}->{x} + $face->{eyes}->{right}->{width} + 1,
	    ymax => $face->{eyes}->{right}->{y} + $face->{eyes}->{right}->{height} + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{eyes}->{right}->{x},
	    ymin => $face->{eyes}->{right}->{y},
	    xmax => $face->{eyes}->{right}->{x} + $face->{eyes}->{right}->{width},
	    ymax => $face->{eyes}->{right}->{y} + $face->{eyes}->{right}->{height},
	    color => $score_color
	);
	# nose mouth chin
	$im->box(
	    xmin => $face->{nose}->{x} + 1,
	    ymin => $face->{nose}->{y} + 1,
	    xmax => $face->{nose}->{x} + 1 * $scale + 1,
	    ymax => $face->{nose}->{y} + 1 * $scale + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{nose}->{x},
	    ymin => $face->{nose}->{y},
	    xmax => $face->{nose}->{x} + 1 * $scale,
	    ymax => $face->{nose}->{y} + 1 * $scale,
	    color => $score_color
	);
	$im->box(
	    xmin => $face->{mouth}->{x} + 1,
	    ymin => $face->{mouth}->{y} + 1,
	    xmax => $face->{mouth}->{x} + $face->{mouth}->{width} + 1,
	    ymax => $face->{mouth}->{y} + $face->{mouth}->{height} + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{mouth}->{x},
	    ymin => $face->{mouth}->{y},
	    xmax => $face->{mouth}->{x} + $face->{mouth}->{width},
	    ymax => $face->{mouth}->{y} + $face->{mouth}->{height},
	    color => $score_color
	);
	$im->box(
	    xmin => $face->{chin}->{x} + 1,
	    ymin => $face->{chin}->{y} + 1,
	    xmax => $face->{chin}->{x} + 1 * $scale + 1,
	    ymax => $face->{chin}->{y} + 1 * $scale + 1,
	    color => $black
	);
	$im->box(
	    xmin => $face->{chin}->{x},
	    ymin => $face->{chin}->{y},
	    xmax => $face->{chin}->{x} + 1 * $scale,
	    ymax => $face->{chin}->{y} + 1 * $scale,
	    color => $score_color
	);
	$im->box(
	    xmin => $face->{face}->{x} + $face->{face}->{width} + 2 + 1,
	    ymin => $face->{face}->{y} + 1,
	    xmax => $face->{face}->{x} + $face->{face}->{width} + 2 + 8 * $scale + 1,
	    ymax => $face->{face}->{y} + 8 * $scale + 1,
	    color => $black);
	$im->box(
	    xmin => $face->{face}->{x} + $face->{face}->{width} + 2,
	    ymin => $face->{face}->{y},
	    xmax => $face->{face}->{x} + $face->{face}->{width} + 2 + 8 * $scale,
	    ymax => $face->{face}->{y} + 8 * $scale,
	    color => $face->{hair_color},
	    filled => 1);
	
    }
    

    return 0;
}
