use 5.008008;
use strict;
use warnings;
use Imager;
require Exporter;

our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw(
detect_animeface
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
detect_animeface
);

our $VERSION = '1.01';
require XSLoader;
XSLoader::load('Imager::AnimeFace', $VERSION);

    
sub detect_animeface
{
    my ($imager, %options) = @_;
    my @results = ();
    my $faces = [];
    my $step = $options{step} || 4.0;
    my $scale_factor = $options{scale_factor} || 1.095;
    my $window = $options{window} || 42.592;

    if ($window < 16.0) {
	$window = 42.592;
    }
    if ($scale_factor < 1.01) {
	$scale_factor = 1.095;
    }
    if ($step < 1.0) {
	$step = 4.0;
    }

    my $nface = Imager::AnimeFace::_detect(
	\@results,
	$imager,
	$step, $scale_factor, $window
    );

    for (my $i = 0; $i < $nface; ++$i) {
	my ($face, $b, $g, $r);
	$face = {};
	# likelihood
	$face->{likelihood} = shift @results;
	# face
	$face->{face} = {};
	$face->{face}->{x} = shift @results;
	$face->{face}->{y} = shift @results;
	$face->{face}->{width} = shift @results;
	$face->{face}->{height} = shift @results;
	$b = shift @results;
	$g = shift @results;
	$r = shift @results;
	$face->{skin_color} = Imager::Color->new(r => $r, g => $g, b => $b);
	$b = shift @results;
	$g = shift @results;
	$r = shift @results;
	$face->{hair_color} = Imager::Color->new(r => $r, g => $g, b => $b);
	# eyes
	$face->{eyes} = {};
	# left eye
	$face->{eyes}->{left}->{x} = shift @results;
	$face->{eyes}->{left}->{y} = shift @results;
	$face->{eyes}->{left}->{width} = shift @results;
	$face->{eyes}->{left}->{height} = shift @results;
	$face->{eyes}->{left}->{colors} = [];
	for (my $j = 0; $j < 4; ++$j) {
	    $b = shift @results;
	    $g = shift @results;
	    $r = shift @results;
	    push(@{$face->{eyes}->{left}->{colors}},
		 Imager::Color->new(r => $r, g => $g, b => $b));
	}
	# right eye
	$face->{eyes}->{right} = {};
	$face->{eyes}->{right}->{x} = shift @results;
	$face->{eyes}->{right}->{y} = shift @results;
	$face->{eyes}->{right}->{width} = shift @results;
	$face->{eyes}->{right}->{height} = shift @results;
	$face->{eyes}->{right}->{colors} = [];
	for (my $j = 0; $j < 4; ++$j) {
	    $b = shift @results;
	    $g = shift @results;
	    $r = shift @results;
	    push(@{$face->{eyes}->{right}->{colors}},
		 Imager::Color->new(r => $r, g => $g, b => $b));
	}
	# nose
	$face->{nose} = {};
	$face->{nose}->{x} = shift @results;
	$face->{nose}->{y} = shift @results;
	$face->{nose}->{width} = 1;
	$face->{nose}->{height} = 1;
	# mouth
	$face->{mouth} = {};
	$face->{mouth}->{x} = shift @results;
	$face->{mouth}->{y} = shift @results;
	$face->{mouth}->{width} = shift @results;
	$face->{mouth}->{height} = shift @results;
	# chin
	$face->{chin} = {};
	$face->{chin}->{x} = shift @results;
	$face->{chin}->{y} = shift @results;
	$face->{chin}->{width} = 1;
	$face->{chin}->{height} = 1;

	push(@{$faces}, $face);
    }

    return $faces;
}


1;
__END__

=head1 NAME

Imager::AnimeFace - Perl extension for Anime-Face

=head1 SYNOPSIS
    
  use Imager;
  use Imager::AnimeFace;
  use Data::Dumper;

  # load imagefile
  my $im = Imager->new();
  $im->read(file => 'anime-imagefile.jpeg');

  # detect
  my $results = detect_animeface($im);

  # show result
  foreach my $face (@{$results}) {
    print Dumper($face);
  }
  

=head1 DESCRIPTION

Imager::AnimeFace is a set of functions
that implement detection and analysis of Anime-Face.

=head2 EXPORT

detect_animeface

=head1 SEE ALSO

Imager(3), Imager::Color(3)

=head1 AUTHOR

http://anime.udp.jp/
<nagadomi@nurs.or.jp>

=cut
